/*
*	author:戴智斌,Steve Tai
*	documentation:
*		option:
*			object_name:物件名稱(型態為string)
*			object_html:可為字串或函式
*				若為字串,此為物件html字串,清單中object呈現的樣子由此option決定
*				(型態為string,必須符合html語法),
*				若為函式,此函式須回傳物件html字串,此函式有一個input,
*				此input為正在加入物件的序號(從1開始數)
*			new_object:function(group){}
*				新增物件後的callback function。
*				group: 該object group的選擇器,用以指定哪個object需設定(型態為string)
*			existing_object_setting:function(group,setting_data){}
*				物件設定方法(型態為使用者自訂函式),必須與object_html相對應
*				group: 該object group的選擇器,用以指定哪個object需設定(型態為string)
*				setting_data: 用以根據此參數資料(格式為使用者自訂),設定object(型態為object)
*		method:
*			請直接參考程式碼註解					
*/
;(function($) {
    
	//宣告list_editor各種public函數
	var methods = {
        init : function(settings){
            
			//setting初始化設定
			var _defaultSettings = {
				object_name:'物件',
				object_html:'<span>'+'物件'+'</span>',
				new_object:function(group){},
				existing_object_setting:function(group,setting_data){}	
			};
			
			//覆寫settings
			var _settings = $.extend(_defaultSettings, settings);
			
			//針對每一個選定的DOM物件做事
			return this.each(function() {
				
				//取得該物件的id
				var id=$(this).attr('id');
				
				//將_settings資訊存入DOM物件中
				$(this).data("list_editor_settings",_settings);
				
				//設定html
				$(this).append(
					'<div id="'+id+'_counter_panel" class="list_editor_counter_panel">'+
						"目前<span id='"+(id+"_counter")+"'>0</span>個"+_settings.object_name+
						'<span id="'+id+'_new_button" class="list_editor_new_button"></span>'+
					'</div>'+
					'<div id="'+id+'_list_panel" class="list_editor_list_panel" id_counter="0">'+
					'</div>'
				);
				
				//設定新增物件按鈕
				$("#"+id+"_new_button").button({
					icons:{
						primary:"ui-icon-plus"
					},
					text:false,
					label:("新增"+_settings.object_name)
				});
				
				//設置新增目的地按鈕的click事件
				$("#"+id+"_new_button").click(function(){
					
					//新增一個物件
					new_object(id,_settings);
				});
			});
        },

		//取得選擇器中第一個list_editor中有幾個物件
        get_counter : function(){
            
			//取得選擇器中第一個list_editor的ID
			var id=this.attr("id");
			
			//回傳物件個數的計數
			return parseInt($("#"+id+"_counter").html(),10);
        },
		
		//檢驗所有list_editor中是否有input與selector未填寫或選擇(有empty value)
        has_empty : function(){
            
			//初始化回傳結果
			var check_result=false;
			
			//針對每個input,select,textarea進行檢驗
			this.find(":input").each(function(){
				if($(this).val()=="" || $(this).val()==undefined){
					check_result=check_result||true;
				}
			});
			
			//回傳檢驗結果
			return check_result;
        },
		
		//將list_editor中所有物件淨空
		remove_all : function(){
			
			//針對選擇器中每個list_editor,進行淨空動作
			return this.each(function(){
				
				//取得該物件的id
				var id=$(this).attr('id');
				
				//將counter設為0
				$("#"+id+"_counter").html(0);
				
				//將panel的id counter設為0
				$('#'+id+'_list_panel').attr("id_counter",0)
				
				//將panel淨空
				$("#"+id+"_list_panel").empty();
			});
		},
		
		//新增多個物件進list_editor
		add_new_object : function(num){
			
			//針對每一個選定的DOM物件做事
			return this.each(function() {
				
				//取得該DOM object的ID
				var id=$(this).attr("id");
				
				//取得list_editor設定
				var _settings=$("#"+id).data("list_editor_settings");
				
				//新增多個物件進list_editor
				for(var i=0;i<num;i++){
					new_object(id,_settings);
				}
			});
		},
		
		//新增多個已存在的物件(已具有資料的物件)進list_editor
		//input:
		//	num: 希望新增多少個物件
		//	setting_data:
		//		1.「用以設定新加入list物件」的資料
		//		2.使用者自訂的資料,必須與使用者自訂的existing_object_setting函數呼應
		//		3.根據setting_data如何設定新物件,由existing_object_setting函式決定
		add_existing_object : function(num,setting_data){
			
			//針對每一個選定的DOM物件做事
			return this.each(function() {
				
				//取得該DOM object的ID
				var id=$(this).attr("id");
				
				//取得list_editor設定
				var _settings=$("#"+id).data("list_editor_settings");
				
				//新增多個物件進list_editor
				for(var i=0;i<num;i++){
				
					//新增一個物件 並取得group的DOM ID
					var group="#"+new_object(id,_settings);
					
					//設定該物件
					_settings.existing_object_setting(group,setting_data);
				}
			});
		}
    };
	
	//新增一個物件
	//input: id(「設定為list editor的DOM物件」的DOM ID)
	function new_object(id,_settings){
		
		//設定物件總數(加1)
		var counter=parseInt($("#"+id+"_counter").html(),10)+1;
		$("#"+id+"_counter").html(counter);
		
		//取得selector的id counter
		var id_counter=$('#'+id+'_list_panel').attr("id_counter");
		$('#'+id+'_list_panel').attr("id_counter",(parseInt(id_counter,10)+1))
		
		//取得HTML字串
		if(typeof(_settings.object_html)=="function"){
			var append_html=_settings.object_html(counter);
		}
		else{
			var append_html=_settings.object_html;
		}
		
		//將selector附加在新增目的地表格中
		$("#"+id+"_list_panel").append(
			"<div "+
				"id='"+id+"_group"+id_counter+"' "+
				"class='"+id+"_group'"+
			">"+
				"<span>"+
					append_html+
				"</span>"+
				"<span "+
					"id='"+id+"_remove_button"+id_counter+"' "+
					"class='list_editor_remove_button'"+
				">"+
				"</span>"+
			"</div>"
		);
		
		//設定_remove_button為jquery按鈕
		$("#"+id+"_remove_button"+id_counter).button({
			icons:{
				primary:"ui-icon-close"
			},
			text:false,
			label:("刪除此"+_settings.object_name)
		});
		
		//設定_remove_button的click事件
		$("#"+id+"_remove_button"+id_counter).click(function(){
			
			//將讓物件刪除
			$("#"+id+"_group"+id_counter).remove();
			
			//設定物件總數(減1)
			var counter=parseInt($("#"+id+"_counter").html(),10)-1;
			$("#"+id+"_counter").html(counter);
		});
		
		//呼叫新增物件的事件
		_settings.new_object("#"+id+"_group"+id_counter);
		
		//回傳object group的DOM ID
		return id+"_group"+id_counter;
	}
	
	//註冊list_editor為jquery函式
	$.fn.list_editor = function(method) {
        
		/* Method calling logic */
        if ( methods[method] ) {
            return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
        } else if ( typeof method === 'object' || ! method ) {
            return methods.init.apply(this, arguments);
        } else {
            $.error( 'Method ' +  method + ' does not exist on jQuery.list_editor' );
        } 
    };
})(jQuery);